--[[

The Game of Life, also known simply as Life, is a cellular automaton devised by the British mathematician John Horton Conway in 1970.

The "game" is a zero-player game, meaning that its evolution is determined by its initial state, requiring no further input. One interacts with the Game of Life by creating an initial configuration and observing how it evolves or, for advanced players, by creating patterns with particular properties.

The universe of the Game of Life is an infinite two-dimensional orthogonal grid of square cells, each of which is in one of two possible states, alive or dead. Every cell interacts with its eight neighbours, which are the cells that are horizontally, vertically, or diagonally adjacent. At each step in time, the following transitions occur:

1.Any live cell with fewer than two live neighbours dies, as if caused by under-population.
2.Any live cell with two or three live neighbours lives on to the next generation.
3.Any live cell with more than three live neighbours dies, as if by overcrowding.
4. Any dead cell with exactly three live neighbours becomes a live cell, as if by reproduction.

The initial pattern constitutes the seed of the system. The first generation is created by applying the above rules simultaneously to every cell in the seed—births and deaths occur simultaneously, and the discrete moment at which this happens is sometimes called a tick (in other words, each generation is a pure function of the preceding one). The rules continue to be applied repeatedly to create further generations.


]]--

-- access to the course library
tmepath = os.getenv("TME_PATH_1_3_0")
package.path = package.path .. ";"..tmepath.."/contrib/courselib/?.lua;./?.lua"

-- access to modules used by the program
local ca     = require ("ca")
local life_patterns   = require ("life_patterns")
local laux   = require ("laux")

DEAD  = 0 
ALIVE = 1
WAIT = 10


life = ca.Model {
    cells = CellularSpace {
        xdim = 30, 
		ydim = 30,
        instance = { state = DEAD },
        neighborhood = {
            strategy = "moore", 
            self     = false,
            wrap     = true
            }
	},
    -- model initialization
    init = function (model)
        ca.insertPattern (model.cells, life_patterns.rpentomino(), 8, 8 )
        model.cells:synchronize()
    end,
    
    changes = function (model)
        forEachCell(model.cells, function(cell)
            neighbors = ca.count_past_neighbors(cell, "state", ALIVE)
            if neighbors < 2 then
                cell.state = DEAD
            elseif neighbors > 3 then
                cell.state = DEAD
            elseif neighbors == 3 and cell.past.state == DEAD then
                cell.state = ALIVE
            end
        end)
        model.cells:synchronize()
        laux.wait (WAIT)
    end,
    
    maps = {theme1 = {{attribute = "state", colors = "Greens", values = {DEAD, ALIVE}}}
       }
}

life:run (1000)

